import videojs from 'video.js';
import window from 'global/window';
import {version as VERSION} from '../package.json';

import './skipIntroButton';

// Default options for the plugin.
const defaults = {
  ranges: [
    {
      name: 'recap',
      label: 'Skip recap'
    },
    {
      name: 'intro',
      label: 'Skip intro'
    }
  ]
};

/**
 * Function to invoke when the player is ready.
 *
 * This is a great place for your plugin to initialize itself. When this
 * function is called, the player will have its DOM and child components
 * in place.
 *
 * @function onPlayerReady
 * @param    {Player} player
 *           A Video.js player object.
 *
 * @param    {Object} [options={}]
 *           A plain object containing options for the plugin.
 *
 * @param    {Array.<{name: String, label: string}>} [options.ranges]
 *           An array of data used to find ranges to skip
 *
 */
const onPlayerReady = (player, options) => {
  const track = player.mediaCuePointsTrack();
  const skipRanges = [];

  player.skipIntroButton = player.addChild('SkipIntroButton');
  player.addClass('vjs-skip-intro');

  player.on('loadedmetadata', () => {
    const cueData = player.mediainfo.cuePoints;

    skipRanges.length = 0;

    // Find cues matching the configured ranges
    options.ranges.forEach(type => {
      const endCueIdx = cueData.findIndex((cue) => cue.name === `end_${type.name}`);
      const startCueIdx = cueData.findIndex((cue) => cue.name === `start_${type.name}`);

      if (endCueIdx > -1) {
        skipRanges.push({
          start: `start_${type.name}`,
          end: `end_${type.name}`,
          time: cueData[endCueIdx].startTime,
          label: type.label
        });
      }

      // Add missing start cues, if needed
      if (endCueIdx > -1 && startCueIdx === -1) {
        videojs.log(`end_${type.name} without start_${type.name}`);
        let previousEnd = 0;

        if (endCueIdx > 0) {
          previousEnd = cueData[endCueIdx - 1].endTime;
        }
        videojs.log(`insert start_${type.name} at ${previousEnd}`);
        track.addCue(new window.VTTCue(
          previousEnd + 0.1,
          cueData[endCueIdx].startTime - 0.1,
          `start_${type.name}`
        ));
      }
    });
  });

  track.addEventListener('cuechange', function(e) {
    track.activeCues.cues_.forEach((cue) => {
      let cueName = cue.text;

      if (cueName === 'CODE') {
        cueName = cue.originalCuePoint.name;
      }
      skipRanges.forEach((data) => {
        if (data.end === cueName) {
          player.skipIntroButton.hide();
        }
        if (data.start === cueName) {
          player.skipIntroButton.update({
            seekTo: data.time,
            label: data.label
          });
          player.skipIntroButton.show();
        }
      });
    });
  });
};

/**
 * A video.js plugin.
 *
 * Adds skip recap/intro/credits etc buttons based on Video Cloud cue points.
 *
 * @function skipIntro
 * @param    {Object} [options={}]
 *           An object of options left to the plugin author to define.
 */
const skipIntro = function(options) {
  this.ready(() => {
    onPlayerReady(this, videojs.mergeOptions(defaults, options));
  });
};

// Register the plugin with video.js.
videojs.registerPlugin('skipIntro', skipIntro);

// Include the version number.
skipIntro.VERSION = VERSION;

export default skipIntro;
